
import gc
import threading
import unittest
import weakref

import os
import sys

if 'TOX_ENV' not in os.environ:
    sys.path.insert(0, '../')

from fibers import Fiber, current


is_pypy = hasattr(sys, 'pypy_version_info')
has_refcount = hasattr(sys, 'getrefcount')


class ArgRefcountTests(unittest.TestCase):

    def test_arg_refs(self):
        if not has_refcount:
            return
        args = ('a', 'b', 'c')
        refcount_before = sys.getrefcount(args)
        g = Fiber(target=lambda *x: None, args=args)
        self.assertEqual(sys.getrefcount(args), refcount_before+1)
        g.switch()
        self.assertEqual(sys.getrefcount(args), refcount_before)
        del g
        self.assertEqual(sys.getrefcount(args), refcount_before)

    def test_kwarg_refs(self):
        if not has_refcount:
            return
        kwargs = {'a': 1234}
        refcount_before = sys.getrefcount(kwargs)
        g = Fiber(lambda **x: None, kwargs=kwargs)
        self.assertEqual(sys.getrefcount(kwargs), refcount_before+1)
        g.switch()
        self.assertEqual(sys.getrefcount(kwargs), refcount_before)
        del g
        self.assertEqual(sys.getrefcount(kwargs), refcount_before)

    def test_threaded_leak(self):
        if is_pypy:
            return
        gg = []
        def worker():
            # only main greenlet present
            gg.append(weakref.ref(current()))
        for i in range(2):
            t = threading.Thread(target=worker)
            t.start()
            t.join()
        current() # update ts_current
        gc.collect()
        for g in gg:
            self.assertTrue(g() is None)

    def test_threaded_adv_leak(self):
        if is_pypy:
            return
        gg = []
        def worker():
            # main and additional *finished* greenlets
            ll = current().ll = []
            def additional():
                ll.append(current())
            for i in range(2):
                Fiber(additional).switch()
            gg.append(weakref.ref(current()))
        for i in range(2):
            t = threading.Thread(target=worker)
            t.start()
            t.join()
        current() # update ts_current
        gc.collect()
        for g in gg:
            self.assertTrue(g() is None)


if __name__ == '__main__':
    unittest.main(verbosity=2)





import gc
import weakref
import unittest

import os
import sys

if 'TOX_ENV' not in os.environ:
    sys.path.insert(0, '../')

import fibers
from fibers import Fiber


class WeakRefTests(unittest.TestCase):
    def test_dead_weakref(self):
        def _dead_fiber():
            g = Fiber(lambda: None)
            g.switch()
            return g
        o = weakref.ref(_dead_fiber())
        gc.collect()
        self.assertEqual(o(), None)

    def test_inactive_weakref(self):
        o = weakref.ref(Fiber())
        gc.collect()
        self.assertEqual(o(), None)


if __name__ == '__main__':
    unittest.main(verbosity=2)





import _continuation
import threading

__all__ = ['Fiber', 'error', 'current']


_tls = threading.local()


def current():
    try:
        return _tls.current_fiber
    except AttributeError:
        _create_main_fiber()
        return _tls.current_fiber


class error(Exception):
    pass


class Fiber(object):
    _cont = None
    _thread_id = None
    _ended = False

    def __init__(self, target=None, args=[], kwargs={}, parent=None):
        def _run(c):
            _tls.current_fiber = self
            try:
                return target(*args, **kwargs)
            finally:
                cont = self._cont
                self._cont = None
                self._ended = True
                parent = self.parent
                while True:
                    if parent is not None and parent._cont is not None and not parent._ended:
                        break
                    parent = parent.parent
                _continuation.permute(cont, parent._cont)

        self._func = _run

        if parent is None:
            parent = current()
        self._thread_id = threading.current_thread().ident
        if self._thread_id != parent._thread_id:
            raise error('parent cannot be on a different thread')
        self.parent = parent

    @classmethod
    def current(cls):
        return current()

    def _get_parent(self):
        return self.__dict__.get('parent', None)
    def _set_parent(self, value):
        if not isinstance(value, Fiber):
            raise TypeError('parent must be a Fiber')
        if value._ended:
            raise ValueError('parent must not have ended')
        if self._thread_id != value._thread_id:
            raise ValueError('parent cannot be on a different thread')
        self.__dict__['parent'] = value

    parent = property(_get_parent, _set_parent)
    del _get_parent, _set_parent

    def switch(self, value=None):
        if self._ended:
            raise error('Fiber has ended')

        curr = current()
        if curr._thread_id != self._thread_id:
            raise error('Cannot switch to a fiber on a different thread')

        if self._cont is None:
            self._cont = _continuation.continulet(self._func)

        try:
            return curr._cont.switch(value=value, to=self._cont)
        finally:
            _tls.current_fiber = curr

    def throw(self, *args):
        if self._ended:
            raise error('Fiber has ended')

        curr = current()
        if curr._thread_id != self._thread_id:
            raise error('Cannot switch to a fiber on a different thread')

        if self._cont is None:
            # Fiber was not started yet, propagate to parent directly
            self._ended = True
            parent = self.parent
            while True:
                if parent is not None and parent._cont is not None and not parent._ended:
                    break
                parent = parent.parent
            return parent.throw(*args)

        try:
            return curr._cont.throw(*args, to=self._cont)
        finally:
            _tls.current_fiber = curr

    def is_alive(self):
        return (self._cont is not None and self._cont.is_pending()) or \
               (self._cont is None and not self._ended)

    def __getstate__(self):
        raise TypeError('cannot serialize Fiber object')


def _create_main_fiber():
    main_fiber = Fiber.__new__(Fiber)
    main_fiber._cont = _continuation.continulet.__new__(_continuation.continulet)
    main_fiber._is_started = True
    main_fiber._thread_id = threading.current_thread().ident
    main_fiber.__dict__['parent'] = None
    _tls.main_fiber = main_fiber
    _tls.current_fiber = main_fiber





import copy
import time
import threading
import unittest

import os
import sys

if 'TOX_ENV' not in os.environ:
    sys.path.insert(0, '../')

import fibers
from fibers import Fiber, current


is_pypy = hasattr(sys, 'pypy_version_info')


class SomeError(Exception):
    pass


def fmain(seen):
    try:
        current().parent.switch()
    except:
        seen.append(sys.exc_info()[0])
        raise
    raise SomeError


def send_exception(g, exc):
    # note: send_exception(g, exc)  can be now done with  g.throw(exc).
    # the purpose of this test is to explicitely check the propagation rules.
    def crasher(exc):
        raise exc
    g1 = Fiber(target=crasher, args=(exc, ), parent=g)
    g1.switch()


class FiberTests(unittest.TestCase):

    def test_simple(self):
        lst = []

        def f():
            lst.append(1)
            current().parent.switch()
            lst.append(3)
        g = Fiber(f)
        lst.append(0)
        g.switch()
        lst.append(2)
        g.switch()
        lst.append(4)
        self.assertEqual(lst, list(range(5)))

    def test_simple2(self):
        lst = []

        def f():
            lst.append(1)
            current().parent.switch()
            lst.append(3)
        g = Fiber(f)
        lst.append(0)
        g.switch()
        lst.append(2)
        g.switch()
        lst.append(4)
        self.assertEqual(lst, list(range(5)))

    def test_two_children(self):
        lst = []

        def f():
            lst.append(1)
            current().parent.switch()
            lst.extend([1, 1])
        g = Fiber(f)
        h = Fiber(f)
        g.switch()
        self.assertEqual(len(lst), 1)
        h.switch()
        self.assertEqual(len(lst), 2)
        h.switch()
        self.assertEqual(len(lst), 4)
        self.assertEqual(h.is_alive(), False)
        g.switch()
        self.assertEqual(len(lst), 6)
        self.assertEqual(g.is_alive(), False)

    def test_two_recursive_children(self):
        lst = []

        def f():
            lst.append(1)
            current().parent.switch()

        def h():
            lst.append(1)
            i = Fiber(f)
            i.switch()
            lst.append(1)
        g = Fiber(h)
        g.switch()
        self.assertEqual(len(lst), 3)

    def test_exception(self):
        seen = []
        g1 = Fiber(target=fmain, args=(seen, ))
        g2 = Fiber(target=fmain, args=(seen, ))
        g1.switch()
        g2.switch()
        g2.parent = g1
        self.assertEqual(seen, [])
        self.assertRaises(SomeError, g2.switch)
        self.assertEqual(seen, [SomeError])

    def test_send_exception(self):
        seen = []
        g1 = Fiber(target=fmain, args=(seen, ))
        g1.switch()
        self.assertRaises(KeyError, send_exception, g1, KeyError)
        self.assertEqual(seen, [KeyError])

#    def test_frame(self):
#        def f1():
#            f = sys._getframe(0)
#            self.assertEqual(f.f_back, None)
#            current().parent.switch(f)
#            return "meaning of life"
#        g = Fiber(f1)
#        frame = g.switch()
#        self.assertTrue(frame is g.t_frame)
#        self.assertTrue(g)
#        next = g.switch()
#        self.assertFalse(g.is_alive())
#        self.assertEqual(next, 'meaning of life')
#        self.assertEqual(g.t_frame, None)

    def test_threads(self):
        success = []

        def f():
            self.test_simple2()
            success.append(True)
        ths = [threading.Thread(target=f) for i in range(10)]
        for th in ths:
            th.start()
        for th in ths:
            th.join()
        self.assertEqual(len(success), len(ths))

    def test_thread_bug(self):
        def runner(x):
            g = Fiber(lambda: time.sleep(x))
            g.switch()
        t1 = threading.Thread(target=runner, args=(0.2,))
        t2 = threading.Thread(target=runner, args=(0.3,))
        t1.start()
        t2.start()
        t1.join()
        t2.join()

    def test_switch_after_thread_of_prev_fiber_exited(self):
        def thread1():
            def fiber1():
                time.sleep(0.4)
            f11 = fibers.Fiber(fiber1)
            f11.switch()
        def thread2():
            time.sleep(0.2)
            def fiber1():
                time.sleep(0.4)
                f22.switch()
            def fiber2():
                pass
            f21 = fibers.Fiber(fiber1)
            f22 = fibers.Fiber(fiber2)
            f21.switch()
        t1 = threading.Thread(target=thread1)
        t2 = threading.Thread(target=thread2)
        t1.start()
        t2.start()
        t1.join()
        t2.join()

    def test_switch_to_another_thread(self):
        data = {}
        error = None
        created_event = threading.Event()
        done_event = threading.Event()

        def foo():
            data['g'] = Fiber(lambda: None)
            created_event.set()
            done_event.wait()
        thread = threading.Thread(target=foo)
        thread.start()
        created_event.wait()
        try:
            data['g'].switch()
        except fibers.error:
            error = sys.exc_info()[1]
        self.assertTrue(error != None, "fibers.error was not raised!")
        done_event.set()
        thread.join()

    def test_threaded_reparent(self):
        data = {}
        created_event = threading.Event()
        done_event = threading.Event()

        def foo():
            data['g'] = Fiber(lambda: None)
            created_event.set()
            done_event.wait()

        def blank():
            current().parent.switch()

        def setparent(g, value):
            g.parent = value

        thread = threading.Thread(target=foo)
        thread.start()
        created_event.wait()
        g = Fiber(blank)
        g.switch()
        self.assertRaises(ValueError, setparent, g, data['g'])
        done_event.set()
        thread.join()

    def test_throw_doesnt_crash(self):
        result = []
        def worker():
            current().parent.switch()
        def creator():
            g = Fiber(worker)
            g.switch()
            result.append(g)
        t = threading.Thread(target=creator)
        t.start()
        t.join()
        self.assertRaises(fibers.error, result[0].throw, SomeError())

    def test_threaded_updatecurrent(self):
        # FIXME (hangs?)
        return
        # released when main thread should execute
        lock1 = threading.Lock()
        lock1.acquire()
        # released when another thread should execute
        lock2 = threading.Lock()
        lock2.acquire()
        class finalized(object):
            def __del__(self):
                # happens while in green_updatecurrent() in main greenlet
                # should be very careful not to accidentally call it again
                # at the same time we must make sure another thread executes
                lock2.release()
                lock1.acquire()
                # now ts_current belongs to another thread
        def deallocator():
            current().parent.switch()
        def fthread():
            lock2.acquire()
            current()
            del g[0]
            lock1.release()
            lock2.acquire()
            current()
            lock1.release()
        main = current()
        g = [Fiber(deallocator)]
        g[0].bomb = finalized()
        g[0].switch()
        t = threading.Thread(target=fthread)
        t.start()
        # let another thread grab ts_current and deallocate g[0]
        lock2.release()
        lock1.acquire()
        # this is the corner stone
        # getcurrent() will notice that ts_current belongs to another thread
        # and start the update process, which would notice that g[0] should
        # be deallocated, and that will execute an object's finalizer. Now,
        # that object will let another thread run so it can grab ts_current
        # again, which would likely crash the interpreter if there's no
        # check for this case at the end of green_updatecurrent(). This test
        # passes if getcurrent() returns correct result, but it's likely
        # to randomly crash if it's not anyway.
        self.assertEqual(current(), main)
        # wait for another thread to complete, just in case
        t.join()

    def test_exc_state(self):
        def f():
            try:
                raise ValueError('fun')
            except:
                exc_info = sys.exc_info()
                t = Fiber(h)
                t.switch()
                self.assertEqual(exc_info, sys.exc_info())
                del t

        def h():
            self.assertEqual(sys.exc_info(), (None, None, None))

        g = Fiber(f)
        g.switch()

    def test_instance_dict(self):
        if is_pypy:
            return
        def f():
            current().test = 42
        def deldict(g):
            del g.__dict__
        def setdict(g, value):
            g.__dict__ = value
        g = Fiber(f)
        self.assertEqual(g.__dict__, {})
        g.switch()
        self.assertEqual(g.test, 42)
        self.assertEqual(g.__dict__, {'test': 42})
        g.__dict__ = g.__dict__
        self.assertEqual(g.__dict__, {'test': 42})
        self.assertRaises(AttributeError, deldict, g)
        self.assertRaises(TypeError, setdict, g, 42)

    def test_deepcopy(self):
        self.assertRaises(TypeError, copy.copy, Fiber())
        self.assertRaises(TypeError, copy.deepcopy, Fiber())

    def test_finished_parent(self):
        def f():
            return 42
        g = Fiber(f)
        g.switch()
        self.assertFalse(g.is_alive())
        self.assertRaises(ValueError, Fiber, parent=g)


if __name__ == '__main__':
    unittest.main(verbosity=2)





import gc
import unittest
import weakref

import os
import sys

if 'TOX_ENV' not in os.environ:
    sys.path.insert(0, '../')

from fibers import Fiber, current


class GCTests(unittest.TestCase):

    def test_circular_fiber(self):
        class circular_fiber(Fiber):
            pass
        o = circular_fiber()
        o.self = o
        o = weakref.ref(o)
        gc.collect()
        self.assertTrue(o() is None)
        self.assertFalse(gc.garbage, gc.garbage)

    def test_dead_circular_ref(self):
        o = weakref.ref(Fiber(current).switch())
        gc.collect()
        self.assertTrue(o() is None)
        self.assertFalse(gc.garbage, gc.garbage)

    def test_inactive_ref(self):
        o = Fiber(lambda: None)
        o = weakref.ref(o)
        gc.collect()
        self.assertTrue(o() is None)
        self.assertFalse(gc.garbage, gc.garbage)

    def test_finalizer_crash(self):
        # This test is designed to crash when active greenlets
        # are made garbage collectable, until the underlying
        # problem is resolved. How does it work:
        # - order of object creation is important
        # - array is created first, so it is moved to unreachable first
        # - we create a cycle between a greenlet and this array
        # - we create an object that participates in gc, is only
        #   referenced by a greenlet, and would corrupt gc lists
        #   on destruction, the easiest is to use an object with
        #   a finalizer
        # - because array is the first object in unreachable it is
        #   cleared first, which causes all references to greenlet
        #   to disappear and causes greenlet to be destroyed, but since
        #   it is still live it causes a switch during gc, which causes
        #   an object with finalizer to be destroyed, which causes stack
        #   corruption and then a crash
        class object_with_finalizer(object):
            def __del__(self):
                pass
        array = []
        parent = current()
        def greenlet_body():
            current().object = object_with_finalizer()
            try:
                parent.switch()
            finally:
                del current().object
        g = Fiber(greenlet_body)
        g.array = array
        array.append(g)
        g.switch()
        del array
        del g
        current()
        gc.collect()


if __name__ == '__main__':
    unittest.main(verbosity=2)





import unittest

import os
import sys

if 'TOX_ENV' not in os.environ:
    sys.path.insert(0, '../')

import fibers
from fibers import Fiber


def switch(val):
    return fibers.current().parent.switch(val)


class ThrowTests(unittest.TestCase):
    def test_class(self):
        def f():
            try:
                switch("ok")
            except RuntimeError:
                switch("ok")
                return
            switch("fail")
        g = Fiber(f)
        res = g.switch()
        self.assertEqual(res, "ok")
        res = g.throw(RuntimeError)
        self.assertEqual(res, "ok")

    def test_val(self):
        def f():
            try:
                switch("ok")
            except RuntimeError:
                val = sys.exc_info()[1]
                if str(val) == "ciao":
                    switch("ok")
                    return
            switch("fail")

        g = Fiber(f)
        res = g.switch()
        self.assertEqual(res, "ok")
        res = g.throw(RuntimeError("ciao"))
        self.assertEqual(res, "ok")

        g = Fiber(f)
        res = g.switch()
        self.assertEqual(res, "ok")
        res = g.throw(RuntimeError, "ciao")
        self.assertEqual(res, "ok")

    def test_kill(self):
        def f():
            try:
                switch("ok")
                switch("fail")
            except Exception as e:
                return e
        g = Fiber(f)
        res = g.switch()
        self.assertEqual(res, "ok")
        res = g.throw(ValueError)
        self.assertTrue(isinstance(res, ValueError))
        self.assertFalse(g.is_alive())

    def test_throw_goes_to_original_parent(self):
        main = fibers.current()

        def f1():
            try:
                main.switch("f1 ready to catch")
            except IndexError:
                return "caught"
            else:
                return "normal exit"

        def f2():
            main.switch("from f2")

        g1 = Fiber(f1)
        g2 = Fiber(target=f2, parent=g1)
        self.assertRaises(IndexError, g2.throw, IndexError)
        self.assertFalse(g2.is_alive())
        self.assertTrue(g1.is_alive())    # g1 is skipped because it was not started

    def test_throw_goes_to_original_parent2(self):
        main = fibers.current()

        def f1():
            try:
                main.switch("f1 ready to catch")
            except IndexError:
                return "caught"
            else:
                return "normal exit"

        def f2():
            main.switch("from f2")

        g1 = Fiber(f1)
        g2 = Fiber(target=f2, parent=g1)
        res = g1.switch()
        self.assertEqual(res, "f1 ready to catch")
        res = g2.throw(IndexError)
        self.assertEqual(res, "caught")
        self.assertFalse(g2.is_alive())
        self.assertFalse(g1.is_alive())

    def test_throw_goes_to_original_parent3(self):
        main = fibers.current()

        def f1():
            try:
                main.switch("f1 ready to catch")
            except IndexError:
                return "caught"
            else:
                return "normal exit"

        def f2():
            main.switch("from f2")

        g1 = Fiber(f1)
        g2 = Fiber(target=f2, parent=g1)
        res = g1.switch()
        self.assertEqual(res, "f1 ready to catch")
        res = g2.switch()
        self.assertEqual(res, "from f2")
        res = g2.throw(IndexError)
        self.assertEqual(res, "caught")
        self.assertFalse(g2.is_alive())
        self.assertFalse(g1.is_alive())


if __name__ == '__main__':
    unittest.main(verbosity=2)




# coding=utf-8

try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension

import glob
import re
import sys


if hasattr(sys, 'pypy_version_info'):
    ext_modules = []
else:
    if sys.platform == 'win32' and '32 bit' in sys.version:
        extra_objects = ['src/switch_x86_msvc.obj']
    elif sys.platform == 'win32' and '64 bit' in sys.version:
        extra_objects = ['src/switch_x64_msvc.obj']
    else:
        extra_objects = []
    ext_modules  = [Extension('fibers._cfibers',
                              sources = glob.glob('src/*.c'),
                              extra_objects=extra_objects,
                             )]


def get_version():
    return re.search(r"""__version__\s+=\s+(?P<quote>['"])(?P<version>.+?)(?P=quote)""", open('fibers/__init__.py').read()).group('version')


setup(name             = "fibers",
      version          = get_version(),
      author           = "Saúl Ibarra Corretgé",
      author_email     = "saghul@gmail.com",
      url              = "http://github.com/saghul/python-fibers",
      description      = "Lightweight cooperative microthreads for Pyhton",
      long_description = open("README.rst").read(),
      packages         = ['fibers'],
      platforms        = ["POSIX", "Microsoft Windows"],
      classifiers      = [
          "Development Status :: 4 - Beta",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: MIT License",
          "Operating System :: POSIX",
          "Operating System :: Microsoft :: Windows",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.0",
          "Programming Language :: Python :: 3.1",
          "Programming Language :: Python :: 3.2",
          "Programming Language :: Python :: 3.3",
          "Programming Language :: Python :: 3.4",
          "Programming Language :: Python :: Implementation :: CPython",
          "Programming Language :: Python :: Implementation :: PyPy",
      ],
      ext_modules  = ext_modules
     )

